@@ -9,7 +9,7 @@ import warnings
 from collections.abc import Iterable
 
 import numpy as np
-from scipy.spatial.distance import pdist, squareform
+from scipy.spatial.distance import cdist
 from astropy.time import Time
 from astropy.coordinates import Angle
 from astropy.utils import iers
@@ -1081,9 +1081,64 @@ def get_lst_for_time(jd_array, latitude, longitude, altitude):
     return lst_array
 
 
-def find_clusters(location_ids, location_vectors, tol):
+def _adj_list(vecs, tol, n_blocks=None):
+    """Identify neighbors of each vec in vecs, to distance tol."""
+    n_items = len(vecs)
+    max_items = 2 ** 10  # Max array size used is max_items**2. Avoid using > 1 GiB
+
+    if n_blocks is None:
+        n_blocks = max(n_items // max_items, 1)
+
+    # We may sort blocks so that some pairs of blocks may be skipped.
+    # Reorder vectors by x.
+
+    order = np.argsort(vecs[:, 0])
+    blocks = np.array_split(order, n_blocks)
+    adj = [{k} for k in range(n_items)]  # Adjacency lists
+    for b1 in blocks:
+        for b2 in blocks:
+            v1, v2 = vecs[b1], vecs[b2]
+            # Check for no overlap, with tolerance.
+            xmin1 = v1[0, 0] - tol
+            xmax1 = v1[-1, 0] + tol
+            xmin2 = v2[0, 0] - tol
+            xmax2 = v2[-1, 0] + tol
+            if max(xmin1, xmin2) > min(xmax1, xmax2):
+                continue
+
+            adj_mat = cdist(vecs[b1], vecs[b2]) < tol
+            for bi, col in enumerate(adj_mat):
+                adj[b1[bi]] = adj[b1[bi]].union(b2[col])
+    return [frozenset(g) for g in adj]
+
+
+def _find_cliques(adj, strict=False):
+    n_items = len(adj)
+
+    loc_gps = []
+    visited = np.zeros(n_items, dtype=bool)
+    for k in range(n_items):
+        if visited[k]:
+            continue
+        a0 = adj[k]
+        visited[k] = True
+        if all(adj[it].__hash__() == a0.__hash__() for it in a0):
+            group = list(a0)
+            group.sort()
+            visited[list(a0)] = True
+            loc_gps.append(group)
+
+    # Require all adjacency lists to be isolated maximal cliques:
+    if strict:
+        if not all(sorted(st) in loc_gps for st in adj):
+            raise ValueError("Non-isolated cliques found in graph.")
+
+    return loc_gps
+
+
+def find_clusters(location_ids, location_vectors, tol, strict=False):
     """
-    Find clusters of vectors (e.g. redundand baselines, times).
+    Find clusters of vectors (e.g. redundant baselines, times).
 
     Parameters
     ----------
@@ -1093,6 +1148,10 @@ def find_clusters(location_ids, location_vectors, tol):
         location vectors, can be multidimensional
     tol : float
         tolerance for clusters
+    strict : bool
+        Require that all adjacency lists be isolated maximal cliques.
+        This ensures that vectors do not fall into multiple clusters.
+        Default: False
 
     Returns
     -------
@@ -1104,37 +1163,10 @@ def find_clusters(location_ids, location_vectors, tol):
     if location_vectors.ndim == 1:
         location_vectors = location_vectors[:, np.newaxis]
 
-    # For each baseline, list all others that are within the tolerance distance.
-    adj_triu_mat = pdist(location_vectors) < tol
-    adj = {}  # Adjacency dictionary
-
-    for bi, col in enumerate(squareform(adj_triu_mat)):
-        col[bi] = True
-        adj[location_ids[bi]] = location_ids[col]
-
-    # The adjacency list defines a set of graph edges.
-    # For each location b0, loop over its adjacency list ai \in adj[b0]
-    #   If adj[b0] is a subset of adj[ai], then ai is in a redundant group with b0
-    loc_gps = []
-    for k in adj.keys():
-        a0 = adj[k]
-        group = [k]
-        for a in a0:
-            if set(a0).issubset(adj[a]) and a not in group:
-                group.append(a)
-        group.sort()
-        loc_gps.append(group)
-
-    # Groups can be different lengths, but we need to take a unique over an axis
-    # to properly identify unique groups
-    # Pad out all the sub-lists to be the same length
-    pad = len(max(loc_gps, key=len))
-    loc_gps = np.array([i + [-1] * (pad - len(i)) for i in loc_gps])
-    # We end up with multiple copies of each redundant group, so remove duplicates
-    loc_gps = np.unique(loc_gps, axis=0).tolist()
-    # remove the dummy pad baselines from each list
-    loc_gps = [[bl for bl in gp if bl != -1] for gp in loc_gps]
+    adj = _adj_list(location_vectors, tol)  # adj = list of sets
 
+    loc_gps = _find_cliques(adj, strict=strict)
+    loc_gps = [np.sort(location_ids[gp]).tolist() for gp in loc_gps]
     return loc_gps
 
 
@@ -1191,7 +1223,13 @@ def get_baseline_redundancies(baselines, baseline_vecs, tol=1.0, with_conjugates
         )
         return bl_gps, vec_bin_centers, lens, baseline_ind_conj
 
-    bl_gps = find_clusters(baselines, baseline_vecs, tol)
+    try:
+        bl_gps = find_clusters(baselines, baseline_vecs, tol, strict=True)
+    except ValueError as exc:
+        raise ValueError(
+            "Some baselines are falling into multiple"
+            " redundant groups. Lower the tolerance to resolve ambiguity."
+        ) from exc
 
     n_unique = len(bl_gps)
     vec_bin_centers = np.zeros((n_unique, 3))
@@ -1200,12 +1238,6 @@ def get_baseline_redundancies(baselines, baseline_vecs, tol=1.0, with_conjugates
         vec_bin_centers[gi] = np.mean(baseline_vecs[inds, :], axis=0)
 
     lens = np.sqrt(np.sum(vec_bin_centers ** 2, axis=1))
-    if np.sum([len(bg) for bg in bl_gps]) > Nbls:
-        raise ValueError(
-            "Some baselines are falling into multiple"
-            " redundant groups. Lower the tolerance to resolve ambiguity."
-        )
-
     return bl_gps, vec_bin_centers, lens
 
 
@@ -1418,13 +1450,13 @@ def or_collapse(
         NOT USED, but kept for symmetry with other collapsing functions.
 
     """
-    if arr.dtype != np.bool:
+    if arr.dtype != np.bool_:
         raise ValueError("Input to or_collapse function must be boolean array")
     out = np.any(arr, axis=axis)
     if (weights is not None) and not np.all(weights == weights.reshape(-1)[0]):
         warnings.warn("Currently weights are not handled when OR-ing boolean arrays.")
     if return_weights:
-        return out, np.ones_like(out, dtype=np.float)
+        return out, np.ones_like(out, dtype=np.float64)
     else:
         return out
 
@@ -1450,13 +1482,13 @@ def and_collapse(
         NOT USED, but kept for symmetry with other collapsing functions.
 
     """
-    if arr.dtype != np.bool:
+    if arr.dtype != np.bool_:
         raise ValueError("Input to and_collapse function must be boolean array")
     out = np.all(arr, axis=axis)
     if (weights is not None) and not np.all(weights == weights.reshape(-1)[0]):
         warnings.warn("Currently weights are not handled when AND-ing boolean arrays.")
     if return_weights:
-        return out, np.ones_like(out, dtype=np.float)
+        return out, np.ones_like(out, dtype=np.float64)
     else:
         return out
 
