@@ -9,7 +9,13 @@ import os
 
 import pytest
 import numpy as np
-import astropy
+
+try:
+    import erfa
+except ModuleNotFoundError:
+    # TODO: This is for backwards compatibility with astropy < 4.2.
+    # When we require 4.2 or greater it should be removed.
+    from astropy import _erfa as erfa
 from astropy.io import fits
 
 from pyuvdata import UVData
@@ -295,7 +301,9 @@ def test_readwriteread_antenna_diameters(tmp_path, casa_uvfits):
     write_file = str(tmp_path / "outtest_casa.uvfits")
 
     # check that if antenna_diameters is set, it's read back out properly
-    uv_in.antenna_diameters = np.zeros((uv_in.Nants_telescope,), dtype=np.float) + 14.0
+    uv_in.antenna_diameters = (
+        np.zeros((uv_in.Nants_telescope,), dtype=np.float64) + 14.0
+    )
     uv_in.write_uvfits(write_file)
     uv_out.read(write_file)
     assert uv_in == uv_out
@@ -354,8 +362,8 @@ def test_readwriteread_missing_info(tmp_path, casa_uvfits):
         ant_hdu = hdu_list[hdunames["AIPS AN"]]
         ant_hdr = ant_hdu.header.copy()
 
-        time_sys = ant_hdr.pop("TIMSYS")
-        ant_hdr["TIMESYS"] = time_sys
+        time_sys = ant_hdr.pop("TIMESYS")
+        ant_hdr["TIMSYS"] = time_sys
         ant_hdr["FRAME"] = "????"
 
         ant_hdu.header = ant_hdr
@@ -424,12 +432,7 @@ def test_readwriteread_error_single_time(tmp_path, casa_uvfits):
 
     with pytest.raises(ValueError) as cm:
         with uvtest.check_warnings(
-            [
-                UserWarning,
-                astropy._erfa.core.ErfaWarning,
-                astropy._erfa.core.ErfaWarning,
-                UserWarning,
-            ],
+            [UserWarning, erfa.core.ErfaWarning, erfa.core.ErfaWarning, UserWarning],
             [
                 "Telescope EVLA is not",
                 'ERFA function "utcut1" yielded 1 of "dubious year (Note 3)"',
@@ -530,7 +533,7 @@ def test_extra_keywords_errors(
     "kwd_names,kwd_values",
     (
         [["bool", "bool2"], [True, False]],
-        [["int1", "int2"], [np.int(5), 7]],
+        [["int1", "int2"], [np.int64(5), 7]],
         [["float1", "float2"], [np.int64(5.3), 6.9]],
         [["complex1", "complex2"], [np.complex64(5.3 + 1.2j), 6.9 + 4.6j]],
         [
@@ -949,3 +952,28 @@ def test_read_ms_write_uvfits_casa_history(tmp_path):
     ms_uv.write_uvfits(testfile, spoof_nonessential=True)
     uvfits_uv.read(testfile)
     assert ms_uv == uvfits_uv
+
+
+def test_cotter_telescope_frame(tmp_path):
+    file1 = os.path.join(DATA_PATH, "1061316296.uvfits")
+    write_file = os.path.join(tmp_path, "emulate_cotter.uvfits")
+    uvd1 = UVData()
+
+    with fits.open(file1, memmap=True) as hdu_list:
+        hdunames = uvutils._fits_indexhdus(hdu_list)
+        vis_hdu = hdu_list[0]
+        ant_hdu = hdu_list[hdunames["AIPS AN"]]
+        ant_hdu.header.pop("FRAME")
+
+        hdulist = fits.HDUList(hdus=[vis_hdu, ant_hdu])
+        hdulist.writeto(write_file, overwrite=True)
+        hdulist.close()
+
+    with uvtest.check_warnings(
+        UserWarning,
+        [
+            "Required Antenna frame keyword not set, but this appears to be a Cotter "
+            "file, setting to ITRF.",
+        ],
+    ):
+        uvd1.read_uvfits(write_file, read_data=False)
